# -*- coding: utf-8 -*-
"""
SCRIPT FOR FIGURE 1 OF GRÜNDEMANN ET AL., 2023, JOURNAL OF HYDROLOGY, DOI=10.1016/j.jhydrol.2023.129558

@author: ggrundemann
"""
#%% load packages
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
import mevpy as mev
from matplotlib.legend_handler import HandlerLine2D
from matplotlib.patches import FancyArrowPatch

# Plot the data

class AnnotationHandler(HandlerLine2D):
    def __init__(self,ms,*args,**kwargs):
        self.ms = ms
        HandlerLine2D.__init__(self,*args,**kwargs)
    def create_artists(self, legend, orig_handle,
                        xdescent, ydescent, width, height, fontsize,
                        trans):
        xdata, xdata_marker = self.get_xdata(legend, xdescent, ydescent,
                                              width, height, fontsize)
        ydata = ((height - ydescent) / 2.) * np.ones(len(xdata), float)
        legline = FancyArrowPatch(posA=(xdata[0],ydata[0]),
                                  posB=(xdata[-1],ydata[-1]),
                                  mutation_scale=self.ms,
                                  **orig_handle.arrowprops)
        legline.set_transform(trans)
        return legline,

# Data to plot
Ta = np.logspace(0.001,4,1333) # return period of the annual maxima
p = 1/Ta 
q = 1-p

def tail_function(value,Ta):
    # if value = 0: exponential function
    # if value = 1: GEV function
    if value == 0: # pure exponential function (not equal to Gumbel, but purely exponential)
        Precip = np.log(Ta)*25
    elif value == 1: # gev estimate
        csi = 0.1
        sigma = 17.35
        mu = 13.81
        Precip = mev.gev_quant(q, csi, sigma, mu)
    else:
        print('value most be 0 or 1')
        
    T1 = Precip[333]
    T10 = Precip[666]
    T100 = Precip[999]
    b = T10 - T1
    a = T100 - b - b - T1
    a2 = T100 - 2*T10 + T1 # check
    h = a/b
    if value == 0:
        print('exponential variables: ')
    elif value == 1:
        print('gev variables: ')
    print('T1 = ' + str(T1))
    print('T10 = ' + str(T10))
    print('T100 = ' + str(T100))
    print('b = ' + str(b))
    print('a = ' + str(a))
    print('a2 = ' + str(a2))
    print('h = ' + str(h))
    return Precip

Pexp = tail_function(0,Ta)
Pgev = tail_function(1,Ta)

#%% make the plot
formatter = mpl.ticker.LogFormatter(10, labelOnlyBase=False) 
c1, c2, c3, c4 = ['#a21626', '#1a5289', '#d24c37', '#3b89ba']

fig, ax1 = plt.subplots(figsize=(14/2.54,12/2.54))
ax1 = plt.subplot(1,1,1)
ax1.plot(Ta,Pgev,color=c1,linestyle='--',label='Extreme value distribution')
ax1.plot(Ta,Pexp,color=c2,linestyle=':',label='Extreme value distribution with an exponential tail')
annotateA = ax1.annotate('',xy=(1000,172.7),xytext=(1000,186.5), label='a', \
              arrowprops=dict(arrowstyle="<->",edgecolor=c3,shrinkA=0,shrinkB=0))
annotateB = ax1.annotate('',xy=(1000,Pexp[666],),xytext=(1000,Pexp[999]),label='b', \
            arrowprops=dict(arrowstyle="<->",edgecolor=c4,shrinkA=0,shrinkB=0))
ax1.annotate('',xy=(100,Pexp[333],),xytext=(100,Pexp[666]),\
            arrowprops=dict(arrowstyle="<->",edgecolor=c4,shrinkA=0,shrinkB=0))
ax1.set_ylabel('Precipitation return level (mm)',fontsize='8')
ax1.set_xlabel('Return period (years)',fontsize='8')
ax1.set_xscale('log')
plt.grid(True, which="major", ls="-")
ax1.set_xlim([7,2000])
ax1.set_ylim([5,225])
ax1.set_yticks((Pexp[333],Pexp[666],Pexp[999],Pgev[999]))
ax1.set_yticklabels(('T1','T10','E100','T100'),fontsize=8)
ax1.xaxis.set_major_formatter(formatter)
ax1.tick_params(axis='both', which='major', labelsize=8)
ax1.set_xticklabels(('','','1','10','100'),fontsize=8)
h, l = ax1.get_legend_handles_labels()
ax1.legend(handles = h + [annotateA, annotateB],
            handler_map={type(annotateA): AnnotationHandler(5)},
            fontsize=8)
plt.tight_layout()
plt.savefig('Fig_01.png',dpi=300)
