# -*- coding: utf-8 -*-
"""
SCRIPT FOR FIGURE 2 OF GRÜNDEMANN ET AL., 2023, JOURNAL OF HYDROLOGY, DOI=10.1016/j.jhydrol.2023.129558

GPEX DATASET DOI = 10.4121/12764429

@author: ggrundemann
"""
#%% load packages
import numpy as np
import xarray as xr
import warnings
warnings.simplefilter('ignore') # filter some warning messages
import matplotlib.pyplot as plt

#%% open GPEX dataset
datapath = '' # PATH TO GPEX DATASET

dsc = xr.open_dataset(datapath + 'GPEX.nc')
mask = ds.mask.values
hyyr = ds.hydroyear.values



#%% figure 2
''' Fig_histogram - difference between calendar and hydrological year T1000 '''
def calculate_weights(lats, lons):
    """
    Return the 2-dimensional weights for the area of interest.
    lats, lons - xarray dataarrays with the latitudes and longitudes
    lats = xarray lat coordinate: dsc.lat
    lons = xarray lon coordinate: dsc.lon
    """
    latr    = np.deg2rad(lats)
    weights = np.cos(latr)
    
    weights2d      = np.zeros([len(weights), 1])
    weights2d[:,0] = weights
    weights2d      = np.tile(weights2d, len(lons))
    return(weights2d)

def round_up_to_odd(f):
    f = int(np.ceil(f))
    return f + 1 if f % 2 == 0 else f

def round_down_to_odd(f):
    f = int(np.ceil(f))
    return f - 1 if f % 2 == 0 else f

def weighted_quantiles(values, quantiles=None, sample_weight=None):
    """ Very close to numpy.percentile, but supports weights.
    NOTE: quantiles should be in [0, 1]!
    :param values: numpy.array with data
    :param quantiles: array-like with many quantiles needed
    :param sample_weight: array-like of the same length as `array`
    :return: dictionary with the weighted quantile statistics for a boxplot
    """
    values    = np.array(values)
    
    if quantiles is None:
        quantiles = 0.005,0.01,0.025,0.05,0.1,0.5,0.9,0.95,0.975,0.99,0.995
    quantiles = np.array(quantiles)
    
    if sample_weight is None:
        sample_weight = np.ones(len(values))
    sample_weight = np.array(sample_weight)
    
    idx           = ~np.isnan(values)
    
    # create 1D arrays of the values, weights and indices 
    values    = values.flatten()
    sample_weight = sample_weight.flatten()
    idx           = idx.flatten()
    
    values        = values[idx]
    sample_weight = sample_weight[idx]
    
    assert np.all(quantiles >= 0) and np.all(quantiles <= 1), \
        'quantiles should be in [0, 1]'

    sorter = np.argsort(values)
    values = values[sorter]
    sample_weight = sample_weight[sorter]

    weighted_quantiles = np.cumsum(sample_weight) - 0.5 * sample_weight
    weighted_quantiles /= np.sum(sample_weight)
    
    vals = np.interp(quantiles, weighted_quantiles, values)
    
    return vals


# parameters for plotting
cgev = '#78c679'
cmev = '#6baed6'
xmax = 15
xmin = -xmax
perc = 1
den = True
itr = 100

weight    = calculate_weights(dsc.lat, dsc.lon) # calculate weights for cell sizes
hyyr_fd   = np.where(hyyr == 1, np.nan, 1) # hydro year excluding Jan, change other months to 1

# calculate the difference for GEV, 
# exclude the cells where the hydrological year equals a calendar year (January)
gevcal    = dsc['q_gev'].loc[dict(tr=itr,calhy=0)].values # GEV calendar year
gevhy     = dsc['q_gev'].loc[dict(tr=itr,calhy=1)].values # GEV hydro year
gevdif    = ((gevcal / gevhy) -1) * 100     # differences cal hydro year in %
gevdif_fd = gevdif * hyyr_fd                # differences excluding Jan
gevdiff   = gevdif_fd[~np.isnan(gevdif_fd)] # flatten, remove nans

# calculate the difference for MEV, 
# exclude the cells where the hydrological year equals a calendar year (January)
mevcal    = dsc['q_mev'].loc[dict(tr=itr,calhy=0)].values # MEV calendar year
mevhy     = dsc['q_mev'].loc[dict(tr=itr,calhy=1)].values # MEV hydro year
mevdif    = ((mevcal / mevhy) -1) * 100     # differences cal hydro year in %
mevdif_fd = mevdif * hyyr_fd                # differences excluding Jan
mevdiff   = mevdif_fd[~np.isnan(mevdif_fd)] # flatten, remove nans

# flatten and remove nans for the weights, same as gevdif_fd (& mevdif_fd)
ww       = weight[~np.isnan(gevdif_fd)] 

# calculate bins, make sure the bins are the same for GEV and MEV
gevmev   = np.vstack([gevdiff, mevdiff]).T   # array with GEV & MEV differences
binmin   = round_down_to_odd(np.min(gevmev)) # minimum bin, round down to odd
binmax   = round_up_to_odd(np.max(gevmev))   # maximum bin, round up to odd

binmin   = binmin - perc/2 
binmax   = binmax + perc/2 
binwidth = perc
bins     = np.arange(binmin, binmax, binwidth)


# quantiles
quant = [0.050, 0.950]

# calculate the weighted quantiles for GEV and MEV
statsg  = weighted_quantiles(gevdiff, sample_weight=ww, quantiles = quant)
statsm  = weighted_quantiles(mevdiff, sample_weight=ww, quantiles = quant)

# plot the figure
fig, ax = plt.subplots(1,1, figsize=(10.5/2.54,10.5/2.54))
ax.hist(gevdiff, bins=bins, weights=ww, density=den, color=cgev, label='GEV', 
        histtype='stepfilled', edgecolor='#238443', alpha=0.5)
ax.hist(mevdiff, bins=bins, weights=ww, density=den, color=cmev, label='MEV', 
        histtype='stepfilled', edgecolor='#2171b5', alpha=0.5)
ax.axvline(x=statsg[0],  color='#238443', linestyle='dashed', dashes=(5,3), 
            label='5th percentile GEV', linewidth=1) 
ax.axvline(x=statsg[1], color='#238443', linestyle='dashed', linewidth=1, 
            label='95th percentile GEV')
ax.axvline(x=statsm[0],  color='#2171b5', linestyle='dashed', dashes=(5,3), 
            label='5th percentile MEV', linewidth=1)
ax.axvline(x=statsm[1], color='#2171b5', linestyle='dashed', linewidth=1, 
            label='95th percentile MEV') 
ax.set_ylabel('Probability (-)',fontsize=8)
ax.set_xlim(xmin, xmax)
ax.set_xticks(np.arange(xmin, xmax+5, 5))
ax.set_xlabel('Difference (%)',fontsize=8)
ax.legend(loc=1,fontsize=8)
ax.tick_params(axis='both', which='major', labelsize=8)
plt.tight_layout()
plt.savefig('Fig_02.pdf', dpi=300, transparant=True)