# -*- coding: utf-8 -*-
"""
SCRIPT FOR FIGURE 3 + 4 OF GRÜNDEMANN ET AL., 2023, JOURNAL OF HYDROLOGY, DOI=10.1016/j.jhydrol.2023.129558

GPEX DATASET DOI = 10.4121/12764429

@author: ggrundemann
"""
#%% load packages
import numpy as np
import pandas as pd
import xarray as xr
import math
import string
import matplotlib as mpl
import matplotlib.pyplot as plt
import matplotlib.cm as cm


#%% open GPEX dataset
datapath = '' # PATH TO GPEX DATASET

ds = xr.open_dataset(datapath + 'GPEX.nc')
mask = ds.mask.values

#%% case study areas, you can add your own
casestudies  = {'USA'       : [495,  510,  860,  920],
                'Brazil'    : [1075, 1095, 1290, 1330],
                'Turkey'    : [495,  510,  2105, 2135],
                'Russia'    : [300,  400,  2300, 2400],
                'Zambia'    : [1025, 1060, 2040, 2065],
                'Portugal'  : [500,  520,  1715, 1730],
                'Guinea'    : [780,  795,  1675, 1720],
                'China'     : [615,  640,  2865, 2900],
                'CAR'       : [800,  880,  1970, 2050],
                'Mongolia'  : [430,  475,  2760, 2860],
                'Bolivia'   : [1080, 1120, 1120, 1140],
                'Namibia'   : [1120, 1170, 1956, 1986],
                'Oman'      : [675,  685,  2350, 2384]}

#%% figure 3
''' Fig_T100 - T100 3 distributions daily prcp - logaritmic '''

# distributions to loop over
evds   = ['gev_estimate', 'pot_estimate', 'mev_estimate']
titles = ['GEV', 'POT', 'MEV']

# parameters for plotting
itr  = 100
idur  = 24
label = 'Precipitation (mm/day)'
locsy = [0.98, 0.68, 0.382]
ticks = [40,50,60,70,80,90,100,200,300,400,500,600,700,800,900,1000]
vmin  = 40
vmax  = 1000

norm      = mpl.colors.LogNorm(vmin=vmin, vmax=vmax)
formatter = mpl.ticker.LogFormatter(10, labelOnlyBase=False)

fig, axes = plt.subplots(nrows=3, ncols=1, constrained_layout=True, 
                          figsize=(15/2.54,22/2.54))
for n, ax in enumerate(axes):
    evd    = evds[n]
    immask = ax.imshow(mask, cmap=cm.gray_r, vmax=2)
    var    = ds[evd].loc[dict(dur=idur, tr=itr)].values
    imvar  = ax.imshow(var, cmap=cm.viridis_r, norm=norm)#,vmin=0)
    ax.set_xticks(ticks=[])
    ax.set_yticks(ticks=[])
    ax.set_title(titles[n],fontsize=9)
    if evd == 'gev_estimate':
        for keys, latlon in casestudies.items():
            if keys == 'Australia':
                ausx = [3253, 3270, 3270, 3258, 3253]
                ausy = [1070, 1070, 1088, 1088, 1070]
                ax.plot(ausx, ausy, c='black', linewidth=0.5)
            elif keys != 'Australia':
                latst, laten, lonst, lonen = latlon
                shapefilex = [lonst, lonen, lonen, lonst, lonst]
                shapefiley = [latst, latst, laten, laten, latst]
                ax.plot(shapefilex, shapefiley, c='black', linewidth=0.5)
    plt.gcf().text(0.01, locsy[n], string.ascii_lowercase[n]+')',fontsize=8)

cbar = fig.colorbar(imvar, format=formatter, norm=plt.Normalize(vmin,vmax),
                    ticks=ticks, orientation='horizontal', pad=0.04,
                    extend='both', aspect=40)
cbar.set_label(label,fontsize=8)
cbar.ax.set_xticklabels(ticks, rotation=35,fontsize=8)
plt.savefig('Fig_03.pdf', 
            dpi=300,
            transparant=True)

#%% Figure 4, same case studies

# functions
def create_buffer(latst, laten, lonst, lonen, values):
    """
    Return values in the pre-determined buffer only, set the rest to nan
    
    latst -- upper-lat 
    laten -- lower-lat
    lonst -- lon-west
    lonen -- lon-east] }
    """
    buffer              = np.load('buffer.npy')
    buffer              = buffer[latst:laten, lonst:lonen]
    buffer[buffer > 1]  = 1
    buffer              = buffer.astype(float)
    buffer[buffer == 0] = np.nan
    buffervalues        = values * buffer
    return buffervalues

def weighted_avg_std(values, lats, lons):
    """
    Return the weighted average and standard deviation.

    values -- Numpy ndarrays with the same shape as lats * lons
    lats   -- xarray latitude coordinates
    lons   -- xarray longitude coordi
    """
    latr    = np.deg2rad(lats)
    weights = np.cos(latr)
    
    weights2d      = np.zeros([len(weights), 1])
    weights2d[:,0] = weights
    weights2d      = np.tile(weights2d, len(lons))
    idx            = ~np.isnan(values)
    if np.isnan(np.sum(values)) == True:
        values = create_buffer(latst, laten, lonst, lonen, values)
        idx    = ~np.isnan(values)
    
    average  = np.average(values[idx], weights=weights2d[idx])
    variance = np.average((values[idx]-average)**2, weights=weights2d[idx])
    return (average, math.sqrt(variance))


Earth_radius  = 6371009 # [m]

# empty variables
means  = np.zeros((len(casestudies.keys()), 4))
stdevs = np.zeros((len(casestudies.keys()), 4))

# the extreme value distributions
evds          = ('gev_estimate', 'pot_estimate', 'mev_estimate','gumbel_estimate')
r = 0                                     # start at row 0
for latlon in casestudies.values():       # loop over the lat/lon combinations
    latst, laten, lonst, lonen = latlon
    latslice                   = slice(latst, laten)
    lonslice                   = slice(lonst, lonen)
    for c in range(0, len(evds)):  
        prcp = ds[evds[c][0:]].isel(dur=3, tr=6, lat=latslice, lon=lonslice)
        means[r,c], stdevs[r,c] = weighted_avg_std(prcp.values, 
                                                    prcp.lat,
                                                    prcp.lon)
    r        = r + 1

sd_mn  = stdevs / means 

dfcs = pd.DataFrame({'area': list(casestudies.keys())})
dfcs['gev_cv']    = sd_mn[:,0]
dfcs['pot_cv']    = sd_mn[:,1]
dfcs['mev_cv']    = sd_mn[:,2]
dfcs['gev_mean']  = means[:,0]
dfcs['pot_mean']  = means[:,1]
dfcs['mev_mean']  = means[:,2]
dfcs['gev_stdev'] = stdevs[:,0]
dfcs['pot_stdev'] = stdevs[:,1]
dfcs['mev_stdev'] = stdevs[:,2]

dfcs = dfcs.sort_values(by=['pot_cv'], ascending=False)

fig, ax = plt.subplots(1, sharex=True, figsize=(18/2.54,7/2.54))
ax.scatter(dfcs['area'], dfcs['gev_cv'], label = 'GEV', color='#78c679')
ax.scatter(dfcs['area'], dfcs['pot_cv'], label = 'POT', color='#fd8d3c')
ax.scatter(dfcs['area'], dfcs['mev_cv'], label = 'MEV', color='#6baed6')
ax.set_ylabel('Coefficient of variation (-)', fontsize=8)
ax.set_xlabel('Case study',fontsize=8)
ax.tick_params(axis='both', which='major', labelsize=8)
ax.tick_params(axis='x',which='major',rotation=25)
ax.set_ylim(0,0.4)
ax.legend(fontsize=8)
plt.tight_layout()
plt.savefig('Fig_04.pdf', dpi=300, transparant=True)
