# -*- coding: utf-8 -*-
"""
SCRIPT FOR FIGURE 5 OF GRÜNDEMANN ET AL., 2023, JOURNAL OF HYDROLOGY, DOI=10.1016/j.jhydrol.2023.129558

GPEX DATASET DOI = 10.4121/12764429

@author: ggrundemann
"""
#%% load packages
import numpy as np
import pandas as pd
import xarray as xr
import matplotlib as mpl
import matplotlib.pyplot as plt


#%% open GPEX dataset
datapath = '' # PATH TO GPEX DATASET

ds = xr.open_dataset(datapath + 'GPEX.nc')
mask = ds.mask.values

#%%

# functions
def calculate_weights(lats, lons):
    """
    Return the 2-dimensional weights for the area of interest.
    lats, lons - xarray dataarrays with the latitudes and longitudes
    lats = xarray lat coordinate: dse.lat
    lons = xarray lon coordinate: dse.lon
    """
    latr    = np.deg2rad(lats)
    weights = np.cos(latr)
    
    weights2d      = np.zeros([len(weights), 1])
    weights2d[:,0] = weights
    weights2d      = np.tile(weights2d, len(lons))
    return(weights2d)

def weighted_mean(values, weights2d):
    """
    Return the weighted average and standard deviation.

    values, weights -- Numpy ndarrays with the same shape.
    """
    
    idx    = ~np.isnan(values)
    
    average  = np.average(values[idx], weights=weights2d[idx])
    return (average)

# set parameters
weight_w     = calculate_weights(ds.lat, ds.lon)
trs          = [1, 10, 39, 100, 1000]
Earth_radius = 6371009 # [m]
extremes     = ['gev', 'pot', 'mev','obs']

extremes     = ['gev_estimate', 'pot_estimate', 'mev_estimate','gumbel_estimate','observed_estimate']
dse = ds.copy()

# empty lists and dataframe to store the data
ddf    = pd.DataFrame()
list_evd = []
list_dur = []
list_trs = []
list_avg = []

mask240 = dse[extremes[0]].sel(dur=240, tr=10).values
mask240[mask240 > 0] = 1
mask240 = np.repeat(mask240[:, :, np.newaxis], 11, axis=2)

# calculate the weighted mean for all EVDs and durations
for evd in extremes:
    for idur in dse.dur.values:
        data = dse[evd].sel(dur=idur)
        data = data * mask240

        for itr in trs:
            if evd == 'observed_estimate' and itr > 40:
                break
            else:
                mean   = weighted_mean(data.sel(tr=itr).values, weight_w)
                list_evd.append(evd)
                list_dur.append(idur)
                list_trs.append(itr)
                list_avg.append(mean)

ddf['evd'] = list_evd
ddf['dur'] = list_dur
ddf['trs'] = list_trs
ddf['avg'] = list_avg

#%%

# plot the figure
xvals = dse.dur.values
cgev = '#78c679'
cpot = '#fd8d3c'
cmev = '#6baed6'
cgum = '#e78ac3'
formatter = mpl.ticker.LogFormatter(10, labelOnlyBase=False) 

tr = [1,10,39]
tr = [10,39,100]

fig, ax = plt.subplots(figsize=(16/2.54,11.5/2.54))
ax.plot(xvals, ddf[(ddf['evd']=='observed_estimate') & (ddf['trs']==tr[0])].avg, 
        color='k', label='T'+str(tr[0])+' Observed', linestyle=':')
ax.plot(xvals, ddf[(ddf['evd']=='observed_estimate') & (ddf['trs']==tr[1])].avg, 
        color='k', label='T'+str(tr[1])+' Observed', linestyle='--')
ax.plot(xvals, ddf[(ddf['evd']=='gev_estimate') & (ddf['trs']==tr[0])].avg,
          color=cgev, label='T'+str(tr[0])+' GEV',   linestyle=':')
ax.plot(xvals, ddf[(ddf['evd']=='gev_estimate') & (ddf['trs']==tr[1])].avg,
          color=cgev, label='T'+str(tr[1])+' GEV',  linestyle='--', dashes=(5,1.5))
ax.plot(xvals, ddf[(ddf['evd']=='gev_estimate') & (ddf['trs']==tr[2])].avg,
          color=cgev, label='T'+str(tr[2])+' GEV')
ax.plot(xvals, ddf[(ddf['evd']=='pot_estimate')  & (ddf['trs']==tr[0])].avg,
          color=cpot, label='T'+str(tr[0])+' POT',     linestyle=':',dashes=(1,3))
ax.plot(xvals, ddf[(ddf['evd']=='pot_estimate')  & (ddf['trs']==tr[1])].avg,
          color=cpot, label='T'+str(tr[1])+' POT',    linestyle='--',dashes=(5,3))
ax.plot(xvals, ddf[(ddf['evd']=='pot_estimate')  & (ddf['trs']==tr[2])].avg,
          color=cpot, label='T'+str(tr[2])+' POT')
ax.plot(xvals, ddf[(ddf['evd']=='mev_estimate') & (ddf['trs']==tr[0])].avg,
          color=cmev, label='T'+str(tr[0])+' MEV',   linestyle=':')
ax.plot(xvals, ddf[(ddf['evd']=='mev_estimate') & (ddf['trs']==tr[1])].avg,
          color=cmev, label='T'+str(tr[1])+' MEV',  linestyle='--')
ax.plot(xvals, ddf[(ddf['evd']=='mev_estimate') & (ddf['trs']==tr[2])].avg,
          color=cmev, label='T'+str(tr[2])+' MEV')
ax.set_yscale('log')
ax.set_xscale('log')
ax.tick_params(axis='both', which='both', labelsize=8)

plt.legend(fontsize=8)
ax.set_xticks(xvals)
ax.set_xticklabels(labels=xvals)#, rotation=30)
ax.tick_params(axis='x', which='minor', bottom=False)
ax.yaxis.set_major_formatter(formatter)
ax.tick_params(axis='y', which='minor')
ax.yaxis.set_minor_formatter(mpl.ticker.FormatStrFormatter("%.f"))
ax.tick_params(axis='both', which='both', labelsize=8)
plt.xlabel('Duration (h)', fontsize=8)
plt.ylabel('Precipitation depth (mm)', fontsize=8)
plt.xlim(2.9, 250)

plt.tight_layout()
plt.savefig('Fig_05.png', dpi=300, transparant=True)