# -*- coding: utf-8 -*-
"""
SCRIPT FOR FIGURE 7 OF GRÜNDEMANN ET AL., 2023, JOURNAL OF HYDROLOGY, DOI=10.1016/j.jhydrol.2023.129558

GPEX DATASET DOI = 10.4121/12764429

@author: ggrundemann
"""
#%% load packages
import numpy as np
import pandas as pd
import xarray as xr
import math
import string
import matplotlib as mpl
import matplotlib.pyplot as plt
import matplotlib.cm as cm


#%% open GPEX dataset
datapath = '' # PATH TO GPEX DATASET

ds = xr.open_dataset(datapath + 'GPEX.nc')
mask = ds.mask.values

#%%
#functions
def calculate_weights(lats, lons):
    """
    Return the 2-dimensional weights for the area of interest.
    lats, lons - xarray dataarrays with the latitudes and longitudes
    dse.lat
    dse.lon
    """
    latr    = np.deg2rad(lats)
    weights = np.cos(latr)
    
    weights2d      = np.zeros([len(weights), 1])
    weights2d[:,0] = weights
    weights2d      = np.tile(weights2d, len(lons))
    return(weights2d)


def weighted_quantiles_boxplot(values, quantiles=None, sample_weight=None):
    """ Very close to numpy.percentile, but supports weights.
    NOTE: quantiles should be in [0, 1]!
    :param values: numpy.array with data
    :param quantiles: array-like with many quantiles needed
    :param sample_weight: array-like of the same length as `array`
    :return: dictionary with the weighted quantile statistics for a boxplot
    """
    values    = np.array(values)
    
    if quantiles is None:
        quantiles = 0.01, 0.25, 0.5, 0.75, 0.99
    quantiles = np.array(quantiles)
    
    if sample_weight is None:
        sample_weight = np.ones(len(values))
    sample_weight = np.array(sample_weight)
    
    idx           = ~np.isnan(values)
    
    # create 1D arrays of the values, weights and indices 
    values    = values.flatten()
    sample_weight = sample_weight.flatten()
    idx           = idx.flatten()
    
    values        = values[idx]
    sample_weight = sample_weight[idx]
    
    assert np.all(quantiles >= 0) and np.all(quantiles <= 1), \
        'quantiles should be in [0, 1]'

    sorter = np.argsort(values)
    values = values[sorter]
    sample_weight = sample_weight[sorter]

    weighted_quantiles = np.cumsum(sample_weight) - 0.5 * sample_weight
    weighted_quantiles /= np.sum(sample_weight)
    
    vals = np.interp(quantiles, weighted_quantiles, values)
    
    dic  = dict(zip(quantiles, vals)) 
    stats = {'med'    : dic[0.5], 
             'q1'     : dic[0.25], 
             'q3'     : dic[0.75],
             'whislo' : dic[0.01],
             'whishi' : dic[0.99],
             'ipr'    : (dic[0.99] - dic[0.01]),
             'mean'   : np.average(values[np.isfinite(values)], weights=sample_weight[np.isfinite(values)])
             }
    return stats

# parameters 
weight_w      = calculate_weights(ds.lat, ds.lon)
Earth_radius  = 6371009 # [m]

mask = ds.pot_estimate.sel(dur=240, tr=10).values
mask[mask > 0] = 1


# empty dataframe and lists to store data in 
evd_list = []
dur_list = []
avg_list = []
med_list = []
q_1_list = []
q_3_list = []
ipr_list = []
wlo_list = []
whi_list = []

for evd in ['gev_estimate','pot_estimate','mev_estimate']:
    print(evd)
    for idur in ds.dur.values:
        T10   = ds[evd].sel(tr=1,   dur=idur)
        T100  = ds[evd].sel(tr=10,  dur=idur)
        T1000 = ds[evd].sel(tr=100, dur=idur)


        b  = T100  - T10
        a = T1000 - 2 * T100 + T10
        h  = a / b
        data = h.values

        data = data * mask

        stats  = weighted_quantiles_boxplot(data, sample_weight=weight_w)
        evd_list.append(evd)
        dur_list.append(idur)
        med_list.append(stats['med'])
        avg_list.append(stats['mean'])
        q_1_list.append(stats['q1'])
        q_3_list.append(stats['q3'])
        wlo_list.append(stats['whislo'])
        whi_list.append(stats['whishi'])
        ipr_list.append(stats['ipr'])

dfbp                 = pd.DataFrame()
dfbp['distribution'] = evd_list
dfbp['duration']     = dur_list
dfbp['mean']         = avg_list
dfbp['med']          = med_list
dfbp['q1']           = q_1_list
dfbp['q3']           = q_3_list
dfbp['whislo']       = wlo_list
dfbp['whishi']       = whi_list
dfbp['ipr']          = ipr_list

# plot the figure
dicgev = dfbp[(dfbp['distribution'] == 'gev_estimate')].to_dict(orient='index')
dicpot = dfbp[(dfbp['distribution'] == 'pot_estimate')].to_dict(orient='index')
dicmev = dfbp[(dfbp['distribution'] == 'mev_estimate')].to_dict(orient='index')

pos0 = np.arange(0.75, 14, 1.7).tolist()
pos1 = np.arange(1.25, 14, 1.7).tolist()
pos2 = np.arange(1.75, 15, 1.7).tolist()
#%%
fig, ax = plt.subplots(figsize=(13.5/2.54,8/2.54))
bp0 = ax.bxp([dicgev[el] for el in dicgev], 
              showfliers=False, 
              positions=pos0,
              patch_artist=True, 
              boxprops=dict(facecolor='#78c679'), 
              medianprops=dict(color='#238443'),
              showmeans=False,#True,
              meanprops=dict(markeredgecolor='#238443',marker='x',markersize=4))
bp1 = ax.bxp([dicpot[el]  for el in dicpot],  
              showfliers=False, 
              positions=pos1,
              patch_artist=True,
              boxprops=dict(facecolor='#fd8d3c'), 
              medianprops=dict(color='#d94701'),
              showmeans=False,#True,
              meanprops=dict(markeredgecolor='#d94701',marker='x',markersize=4))
bp2 = ax.bxp([dicmev[el] for el in dicmev], 
              showfliers=False, 
              positions=pos2,
              patch_artist=True, 
              boxprops=dict(facecolor='#6baed6'), 
              medianprops=dict(color='#2171b5'),
              showmeans=False,#True,
              meanprops=dict(markeredgecolor='#2171b5',marker='x',markersize=4))

ax.set_ylabel('$h_\mathrm{T1-T10-T100}$ (-)', fontsize=8)
ax.set_xlabel('Duration (h)', fontsize=8)
ax.set_xticks(pos1)
ax.hlines(0, -5, 20, linewidths=0.5,linestyle='--',color='grey')
ax.set_xlim(0.2, 14.2)
ax.set_xticklabels(['3','6', '12', '24', '48', '72', '120', '240'],fontsize=8)
ax.legend([bp0["boxes"][0], bp1["boxes"][0], bp2["boxes"][0]],
           ['GEV', 'POT','MEV'], loc='upper right',fontsize=8)
ax.tick_params(axis='both', which='major', labelsize=8)
plt.tight_layout()
plt.savefig('Fig_07.png',dpi=300)